package edu.ucr.runner;

/*
 * Copyright 2020 Renjie Wu
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import com.timeseries.TimeSeries;
import edu.ucr.impl.DtwUpd;
import edu.ucr.util.ProgressBar;
import edu.ucr.util.SquaredDistance;
import edu.ucr.util.TsvReader;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

public class PairsRunner {
    private static final int MEASURE_ROUNDS = 1000;
    private static final int[] FASTDTW_RADIUS = { 10, 20, 30, 40 };

    public static void main(String[] args) throws IOException {
        var labelA = new ArrayList<Integer>();
        var dataA = new ArrayList<double[]>();
        var dataATS = new ArrayList<TimeSeries>();

        var labelB = new ArrayList<Integer>();
        var dataB = new ArrayList<double[]>();
        var dataBTS = new ArrayList<TimeSeries>();

        TsvReader.readTsv(args[0], dataA, dataATS, labelA);
        TsvReader.readTsv(args[1], dataB, dataBTS, labelB);

        var cDtwPercent = Double.parseDouble(args[2]);

        try (var outputWriter = new PrintWriter(args[3])) {
            outputWriter.print("Label");
            for (int radius : FASTDTW_RADIUS) {
                outputWriter.format(",FastDTW%dDist", radius);
            }
            outputWriter.print(",cDTWDist");
            for (int radius : FASTDTW_RADIUS) {
                outputWriter.format(",FastDtw%dTime", radius);
            }
            outputWriter.println(",cDTWTime");

            for (int i = 0; i < labelA.size(); i++) {
                long startTime, elapsed;
                var fastDtwTotal = new long[FASTDTW_RADIUS.length];
                long cDtwTotal = 0;
                var fastDtwDist = new double[FASTDTW_RADIUS.length];
                double cDtwDist = -1.0;

                for (int j = 0; j < MEASURE_ROUNDS; j++) {
                    for (int k = 0; k < FASTDTW_RADIUS.length; k++) {
                        startTime = System.nanoTime();
                        fastDtwDist[k] = com.dtw.FastDTW.getWarpDistBetween(dataATS.get(i), dataBTS.get(i), FASTDTW_RADIUS[k], new SquaredDistance());
                        elapsed = System.nanoTime() - startTime;
                        fastDtwTotal[k] += elapsed;
                    }

                    startTime = System.nanoTime();
                    cDtwDist = DtwUpd.computeDtw(dataA.get(i), dataB.get(i), (int) Math.ceil(dataA.get(i).length * cDtwPercent / 100.0));
                    elapsed = System.nanoTime() - startTime;
                    cDtwTotal += elapsed;

                    ProgressBar.show(i + 1, j + 1, MEASURE_ROUNDS);
                }

                outputWriter.format("%d", labelA.get(i));
                for (int k = 0; k < FASTDTW_RADIUS.length; k++) {
                    outputWriter.format(",%.6f", fastDtwDist[k]);
                }
                outputWriter.format(",%.6f", cDtwDist);
                for (int k = 0; k < FASTDTW_RADIUS.length; k++) {
                    var avg = fastDtwTotal[k] * 1.0 / MEASURE_ROUNDS;
                    outputWriter.format(",%.6f", avg);
                }

                var cDtwAvg = cDtwTotal * 1.0 / MEASURE_ROUNDS;
                outputWriter.format(",%.6f", cDtwAvg);

                outputWriter.println();
                outputWriter.flush();

                for (int k = 0; k < FASTDTW_RADIUS.length; k++) {
                    var avg = fastDtwTotal[k] * 1.0 / MEASURE_ROUNDS;
                    System.out.format(", fastDTW%d avg.: %.6f ns", FASTDTW_RADIUS[k], avg);
                }
                System.out.format(", cDTW avg.: %.6f ns\n", cDtwAvg);
            }
        }

    }
}
